package org.eBide.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
public class Product implements Serializable {

	private static final long serialVersionUID = 2515130016314519409L;
	
	public enum State {BID, FINISHED, SOLD, PAID, SENT, RECEIVED };

	// ---------- Private attributes -----------
	private int id;
	
	private String title;
	private String searchString;
	private String description;
	
	private int quantity;
	private long startPrice;
	private Date startDate;
	private Date endDate;
	
	private User vendor;
	private Category category;
	
	private State state;
	
//	private Byte[] photo;
	
	// ---------- Constructors -----------------
	public Product() {}
	public Product(String title, String description, int quantity,
			long startPrice, Date startDate, Date endDate) {
		this.title = title;
		this.searchString = title.toLowerCase();
		this.description = description;
		this.quantity = quantity;
		this.startPrice = startPrice;
		this.startDate = startDate;
		this.endDate = endDate;
		this.state = State.BID;
	}
	// ---------- Public methods ---------------
	// ---------- Getters and Setters ----------
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public String getTitle() {
		return title;
	}
	public void setTitle(String title) {
		this.title = title;
		this.searchString = title.toLowerCase();
	}
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	public int getQuantity() {
		return quantity;
	}
	public void setQuantity(int quantity) {
		this.quantity = quantity;
	}
	public long getStartPrice() {
		return startPrice;
	}
	public void setStartPrice(long startPrice) {
		this.startPrice = startPrice;
	}
	
	@Temporal(TemporalType.TIMESTAMP)
	public Date getStartDate() {
		return startDate;
	}
	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}
	
	@Temporal(TemporalType.TIMESTAMP)
	public Date getEndDate() {
		return endDate;
	}
	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}
	
	@ManyToOne
	public User getVendor() {
		return vendor;
	}
	public void setVendor(User vendor) {
		this.vendor = vendor;
	}
	@ManyToOne(fetch=FetchType.EAGER)
	public Category getCategory() {
		return category;
	}
	public void setCategory(Category category) {
		this.category = category;
	}
	
	@Enumerated(EnumType.ORDINAL)
	public State getState() {
		return state;
	}
	public void setState(State state) {
		this.state = state;
	}
	public void setSearchString(String searchString) {
		this.searchString = searchString;
	}
	public String getSearchString() {
		return searchString;
	}

//	@Lob
//	public Byte[] getPhoto() {
//		return photo;
//	}
//	public void setPhoto(Byte[] photo) {
//		this.photo = photo;
//	}

}
