package org.eBide.sessions;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.eBide.model.Bid;
import org.eBide.model.Product;
import org.eBide.model.User;

@SuppressWarnings("unchecked")
@Stateless
public class ManageBid implements ManageBidRemote {

	@PersistenceContext
	private EntityManager em;

	@Override
	public Bid getBestBid(User user, Product product) {
		User u = em.merge(user);
		Product p = em.merge(product);
		Query q = em.createQuery("SELECT b FROM Bid b WHERE b.bidder = :bd " +
													  "AND b.product = :p " +
													  "ORDER BY b.amount DESC")
					.setParameter("bd", u)
					.setParameter("p", p)
					.setMaxResults(1);
		List<Bid> bids = new ArrayList<Bid>();
		bids.addAll(q.getResultList());
		Bid bid = null;
		if(bids.size() > 0) {
			bid = bids.get(0);
		}
		return bid;
	}

	@Override
	public Bid getBestBid(Product product) {
		Product p = em.merge(product);
		Query q = em.createQuery("SELECT b FROM Bid b WHERE b.product = :p " +
													  "ORDER BY b.amount DESC");
		q.setParameter("p", p);
		q.setMaxResults(1);
		List<Bid> results = q.getResultList();
		Bid bid = null;
		if(results.size() > 0) {
			bid = results.get(0);
		}
		return bid;
	}

	@Override
	public Bid addBid(int amount, Product p, User u) {
		// We have to re-attach the product and the user
		Product pt = em.merge(p);
		User bidder = em.merge(u);
		Bid b = new Bid(amount, pt, bidder);
		em.persist(b);
		u.addBid(b);
		em.merge(bidder);
		return b;
	}
	
	@Override
	public List<Bid> getBidsOnProduct(Product product) {
		Product p = em.merge(product);
		Query q = em.createQuery("SELECT b FROM Bid b WHERE b.product = :p");
		q.setParameter("p", p);
		
		return q.getResultList();
	}
}
