package org.eBide.sessions;
import java.util.List;

import javax.ejb.Remote;

import org.eBide.model.Category;
import org.eBide.model.Product;

@Remote
public interface ManageCategoryRemote {
	
	/** Create a new category.
	 * @param name The name of the category.
	 * @param parent The parent of the Category, null if top level.
	 * @return The new category created
	 */
	public Category create(String name, Category parent);
	
	/** List all product of a given category.
	 * 
	 * @param id The id of the category.
	 * @param recursive If true, also list the products of the subcategories.
	 * (not implemented)
	 * @return The list of the products.
	 */
	public List<Product> listProducts(int id, boolean recursive);
	
	/** Get a category from its id.
	 * 
	 * @param id The Category's Id.
	 * @return The category
	 */
	public Category getCategoryFromId(int id);
	
	
	/** List all categories registered in database.
	 *
	 * @return The list of categories
	 */
	public List<Category> listCategories();
	
}
