package org.eBide.sessions;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.eBide.model.Bid;
import org.eBide.model.Category;
import org.eBide.model.Product;
import org.eBide.model.User;
import org.eBide.model.Product.State;

/**
 * Session Bean implementation class ManageProduc
 */
@SuppressWarnings("unchecked")
@Stateless
public class ManageProduct implements ManageProductRemote {

	@PersistenceContext
	private EntityManager em;

	@Override
	public  Product addProduct(Product p) {
		return null;
	}

	@Override
	public List<Product> getLastProductList(int n) {
		return em.createQuery("SELECT p FROM Product p " +
								"WHERE p.state = :s " +
								"ORDER BY p.id DESC")
					.setParameter("s", State.BID)
					.setMaxResults(n)
					.getResultList();
	}

	@Override
	public List<Product> getMyBiddenProductList(User owner) {
		User u = em.merge(owner);
		return em.createQuery("SELECT DISTINCT p FROM Product p, Bid b " +
				"WHERE b.product = p AND b.bidder = :u AND p.state = :s")
				.setParameter("u", u)
				.setParameter("s", State.BID)
				.getResultList();
	}
	
	@Override
	public List<Product> getMySoldProductList(User owner) {
		return null;
	}

	@Override
	public void modify(Product p) {
		em.merge(p);
	}

	@Override
	public Product getProductFromId(int id) {
		return em.find(Product.class, id);
	}

	@Override
	public Product createProduct(String title, String description,
			int quantity, long startPrice, Date startDate, Date endDate,
			Category cat, User user) {
		User u = em.merge(user);
		Category c = em.merge(cat);
		Product p = new Product(title, description, quantity,
								startPrice, startDate, endDate);
		em.persist(p);
		u.addSell(p);
		p.setVendor(u);
		p.setCategory(c);
		
		em.merge(u);
		return p;
	}

	@Override
	public void updateProduct() {
		// Used to check if a product has a bid
		String query = "SELECT b FROM Bid b WHERE b.product = :p";
		
		List<Product> products = em.createQuery("SELECT p FROM Product p WHERE p.endDate < :d " +
																		"AND p.state = :s")
									.setParameter("d", new Date())
									.setParameter("s", State.BID)
									.getResultList();
		
		for(Product p : products) {
			if(em.createQuery(query).setParameter("p", p).getResultList().size() > 0) {
				p.setState(State.SOLD);
			} else {
				p.setState(State.FINISHED);
			}
			
			em.merge(p);
		}
	}

	@Override
	public List<Product> lookForProduct(String title) {
		return em.createQuery("SELECT p FROM Product p WHERE p.searchString LIKE '%"+title+"%' " +
														"AND p.state = :s")
			    .setParameter("s", State.BID)
				.getResultList();
		
	}
	
	@Override
	public List<Product> getNotSold(User u) {
		List<Product> products = new ArrayList<Product>();
		
		u = em.merge(u);
		
		for(Product p : u.getSells()) {
			if(p.getState().equals(State.FINISHED))  {
				products.add(p);
			}
		}
		
		return products;
	}

	@Override
	public List<Product> getReceived(User u) {
		u = em.merge(u);
		
		return em.createQuery("SELECT p FROM Product p, Bid b WHERE b.product = p" +
												" AND b.bidder = :u AND p.state = :s")
				.setParameter("u", u)
				.setParameter("s", State.RECEIVED)
				.getResultList();
	}

	@Override
	public List<Bid> getToBePaid(User u) {
		System.out.println(u.getLogin());
		u = em.merge(u);
		
		return em.createQuery("SELECT b FROM Bid b, Product p WHERE b.product = p" +
												" AND b.bidder = :u AND p.state = :s")
				 .setParameter("u", u)
				 .setParameter("s", State.SOLD)
				 .getResultList();
	}

	@Override
	public List<Product> getToBeReceived(User u) {
		u = em.merge(u);
		return em.createQuery("SELECT p FROM Product p, Bid b WHERE b.product = p" +
								" AND b.bidder = :u AND p.state = :s")
								.setParameter("u", u)
								.setParameter("s", State.SENT)
								.getResultList();
	}

	@Override
	public List<Product> getToBeSent(User u) {
		List<Product> products = new ArrayList<Product>();
		
		u = em.merge(u);
		
		for(Product p : u.getSells()) {
			if(p.getState().equals(State.PAID))  {
				products.add(p);
			}
		}
		
		return products;
	}
	
	@Override
	public List<Product> getSent(User u) {
		List<Product> products = new ArrayList<Product>();
		
		u = em.merge(u);
		
		for(Product p : u.getSells()) {
			if(p.getState().equals(State.SENT))  {
				products.add(p);
			}
		}
		
		return products;
	}

	@Override
	public List<Bid> getMyBoughtProductList(User owner) {
		owner = em.merge(owner);
		return em.createQuery("SELECT b FROM Bid b WHERE b.bidder = :u " +
				"AND NOT (b.product.state = :b OR b.product.state = :f)")
				.setParameter("u", owner)
				.setParameter("b", State.BID)
				.setParameter("f", State.FINISHED)
				.getResultList();
	}
	
	@Override
	public List<Bid> getMyBoughtProductList(String login, int n) {
		User owner = em.find(User.class, login);
		return em.createQuery("SELECT b FROM Bid b WHERE b.bidder = :u " +
				"AND NOT (b.product.state = :b OR b.product.state = :f) " +
				"ORDER BY b.product.id")
				.setParameter("u", owner)
				.setParameter("b", State.BID)
				.setParameter("f", State.FINISHED)
				.setMaxResults(n)
				.getResultList();
	}
	
	@Override
	public void remove(Product p) {
		p = em.merge(p);
		User u = null;
		List<User> users = em.createQuery("SELECT u FROM User u WHERE :p IN u.sells")
		   .setParameter("p", p)
		   .setMaxResults(1)
		   .getResultList();
		if(users.size() > 0) {
			u = users.get(0);
			u.getSells().remove(p);
		}
	}

	@Override
	public List<Product> getSoldProductsInState(User vendor, State state) {
		User u = em.merge(vendor);
		
		return em.createQuery("SELECT p FROM Product p WHERE p.vendor = :v "
													+ "AND p.state = :s")
				.setParameter("v", u)
				.setParameter("s", state)
				.getResultList();
	}

	@Override
	public List<Product> getLastProductsSold(String login, int n) {
		User vendor = em.find(User.class, login);
		return em.createQuery("SELECT p FROM Product p WHERE p.vendor = :p "
														+"AND p.state = :s "
														+"ORDER BY p.id DESC")
				 .setParameter("p", vendor)
				 .setParameter("s", State.BID)
				 .setMaxResults(n)
				 .getResultList();
	}

	@Override
	public List<Product> getBoughtProductsInState(User buyer, State state) {
		User b = em.merge(buyer);
		
		return em.createQuery("SELECT p FROM Product p, Bid b "
									+ "WHERE b.product = p AND b.bidder = :c "
									+ "AND p.state = :s")
				.setParameter("c", b)
				.setParameter("s", state)
				.getResultList();
	
	}

}
