package org.eBide.sessions;

import java.util.Date;
import java.util.List;

import javax.ejb.Remote;

import org.eBide.model.*;
import org.eBide.model.Product.State;

@Remote
public interface ManageProductRemote {

	/**Get the sorted -by start date- list of the n last added Products.
	 * @param n : number of products expected
	 * @return The list of products
	 */
	public List<Product> getLastProductList(int n);
	
	/** Get the list of the n-last products sold by User with id <i>id</i>.
	 * 
	 * @param login Vendor's login
	 * @param n Number of products to return
	 * @return The list of <= n last products sold by vendor.
	 */
	public List<Product> getLastProductsSold(String login, int n);
	
	/**Get the list of Product on which the User has a bid.
	 * @param owner : the user whose bidden products are seeked
	 * @return The list of product
	 */
	public List<Product> getMyBiddenProductList(User owner);
	
	/** Get the list of the winning bids the user has on products*/
	public List<Bid> getMyBoughtProductList(User owner);
	
	/** Get the list of the winning bids the user has on products*/
	public List<Bid> getMyBoughtProductList(String login, int n);
	
	/** Get the list of expired products from user <i>u</i> with no bid. */
	public List<Product> getNotSold(User u);

	/** Get the list of products an User <i>u</i> has to be paid. */
	public List<Bid> getToBePaid(User u);
	
	/** Get the list of products an User <i>u</i> has to be sent.
	 * 
	 * @param u
	 * @return
	 * @deprecated
	 */
	public List<Product> getToBeSent(User u);
	
	/** Get the list of products an User <i>u</i> has sent.
	 * 
	 * @param u
	 * @return
	 * @deprecated
	 * 
	 */
	public List<Product> getSent(User u);
	
	/** Get the list of products an User <i>u</i> has to be received.
	 * 
	 * @param u
	 * @return
	 * @deprecated
	 */
	public List<Product> getToBeReceived(User u);
	
	/** Get the list of products an User <i>u</i> has received.
	 * 
	 * @param u
	 * @return
	 * @deprecated
	 */
	public List<Product> getReceived(User u);
	
	/** Get the list of products sold by <i>vendor</i> and in state <i>state</i>.
	 * 
	 * @param vendor The seller of the products.
	 * @param state The states of the products.
	 * @return Product p with p in vendor.sells and p.state = state
	 */
	public List<Product> getSoldProductsInState(User vendor, State state);
	
	/** Get the list of products bought by <i>buyer</i> in <i>state</i>
	 * 
	 * @param buyer The buyer
	 * @param state The state of the bid
	 * @return The list of products
	 */
	public List<Product> getBoughtProductsInState(User buyer, State state);
	
	/**Get the sorted -by end date- list of Product sold by the User.
	 * @param owner : the seller
	 * @return The list of products
     * @deprecated
	 */
	public List<Product> getMySoldProductList(User owner);
	
	
	/** Return the Product whose id is id!
	 * @param id : the product's id
	 * @return The product
	 */
	public Product getProductFromId(int id);
	
	/** Persist the product in database.
	 * @param p
	 * @deprecated
	 */
	public Product addProduct(Product p);
	
	/** Create a Product from the mandatory data
	 * @return : the created Product
	 */
	public Product createProduct(String title, String description,
			int quantity, long startPrice, Date startDate, Date endDate,
			Category cat, User user);	
	
	/** Modify the information about a Product. */
	public void modify(Product p);
	
	/** Update all products whose state is BID. */
	public void updateProduct();
	
	/** Look for products whose title starts with <i>title</i>. */
	public List<Product> lookForProduct(String title);
	
	/** Delete a product from database. */
	public void remove(Product p);

}
