package org.eBide.sessions;
import java.util.Date;

import javax.ejb.Remote;

import org.eBide.model.User;

@Remote
public interface ManageUserRemote {
	
	/** Creates an User from mandatory data.
	 * @return The created User
	 * @deprecated
	 */
	public User createUser(String name, String firstName, Date birthdate, String civility,
			String address, String city, String zipCode, String country, String login,
			String password, String mail);
	
	/**Adds an user to the DB
	 * @param user : the user to be added
	 */
	public User addUser(User user);
	
	/** Modify information about a User. */
	public void modify(User user);
	
	
	/** Check if a login is already used by an user.
	 * @param login : the login to be checked
	 * @return
	 */
	public boolean loginUsed(String login);
	
	/** Get the user, providing his login and passwords.
	 * If the user doesn't exist or the password is not correct: null
	 * @param login
	 * @param password
	 * @return
	 */
	public User getUser(String login,String password);
	
	/** Check login.
	 * Check if a user with <i>login</i> and <i>password</i> exists.
	 * @param login The login of the user
	 * @param password The crypted password
	 * @return True if there is a match, false otherwise
	 */
	public boolean checkLoggin(String login, String password);
	
	/** Retrieve password for a User.
	 * Changes the password, and return its new value
	 * @param login The Login of the user
	 * @param mail The mail of the user
	 * @return The new value of the User's password
	 */
	public String retrievePassword(String login, String mail);
	
}
