package org.eBide.controller;

import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.component.html.HtmlCommandButton;
import javax.faces.context.FacesContext;

import org.eBide.model.Bid;
import org.eBide.model.Category;
import org.eBide.model.Product;
import org.eBide.model.User;
import org.eBide.model.Product.State;

public class ProductController extends SuperController{
	
	//The "Product" attributes: used for the creation, etc...
	private Product product;
	private String title;
	private String description;
	private int quantity;
	private long startPrice;
	private Date startDate;
	private Date endDate;
	private int categoryId;
	
	//Test attribute: used to renew a finished and unattributed sell.
	private boolean reNewSell = false;
	
	public ProductController(){
		product = new Product();
	}

		
	//Used to determine the table right-side button text given the product state
	//Applied for the Vendor's table (mysell.jsp...)
	public String actionFromStateVendor(Product p){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ResourceBundle bundle = ResourceBundle.getBundle("org.eBide.message.mysell", facesContext.getViewRoot().getLocale());
		
		if(p==null){
			return "Error";
		}
		else if (p.getState()==State.BID){
			return bundle.getString("editSell");
		}
		else if (p.getState()==State.FINISHED){
			return bundle.getString("renewsell");
		}
		else if (p.getState()==State.SOLD || p.getState()==State.PAID){
			return bundle.getString("concludeSell");
		}
		else{
			return "";
		}
	}
	
	//Used to determine the table right-side button text given the product state
	//Applied for the Buyer's table (mybag.jsp...)
	public String actionFromStateBuyer(Product p){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ResourceBundle bundle = ResourceBundle.getBundle("org.eBide.message.mybag", facesContext.getViewRoot().getLocale());
		
		if(p==null){
			return "Error";
		}
		else if (p.getState()==State.SOLD){
			return bundle.getString("payproduct");
		}
		else if (p.getState()==State.PAID){
		
			return bundle.getString("wait");
		}
		else if (p.getState()==State.SENT){
			return bundle.getString("ack");
		}
		else{
			return "";
		}
	}

	//Create a new Sell
	//Return a redirection String
	public String actionCreateSell(){
		User user = getUserController().getUser();
		
		Category category = categoryManager.getCategoryFromId(categoryId);
		if(!reNewSell){
			// New Product
			productManager.createProduct(title, description, quantity,
					startPrice, startDate, endDate, category, user);
		}
		else{
			// Modify the expired Sell
			product.setDescription(description);
			product.setEndDate(endDate);
			product.setQuantity(quantity);
			product.setStartDate(startDate);
			product.setStartPrice(startPrice);
			product.setState(State.BID);
			product.setTitle(title);
			product.setCategory(category);
			productManager.modify(product);	
		}
		
		reNewSell = false;
		getUserController().setUser(userManager.getUser(user.getLogin(), user.getPassword()));
		user = getUserController().getUser();
		
		product = new Product();
		title = new String();
		description = new String();
		quantity = 0;
		startPrice = 0;
		
		return "mysell";
	}
	
	//"Select" a product
	//Set the ProductController attribute in order to "load" the product for a method.
	//Ex: The user select a product on mysell.jsp. The required informations will be loaded in ProductController,
	//and displayed in editsell.jsp.
	//Return a redirection String, depending of the product state
	public String actionSelectSell(){
		try{
			int id = Integer.parseInt(getParam("product"));
			product = productManager.getProductFromId(id);
		}
		catch(NumberFormatException e){
		}
		if(product.getState()==State.BID)
			return "editsell";
		else if(product.getState()==State.FINISHED){
				reNewSell = true;
				title=product.getTitle();
				description=product.getDescription();
				quantity=product.getQuantity();
				startPrice=product.getStartPrice();
				Date today = new Date();
				Date tommorow = new Date(today.getTime() + 86400000);
				setStartDate(today);
				setEndDate(tommorow);
				return "newsell";
		}
		else {
			return "concludesell";
		}
	}
	
	//Cf actionSelectSell: load a product and return the appropritate redirection
	//Return redirection String
	public String actionSelectBoughtProduct(){
		try{
			int id = Integer.parseInt(getParam("product"));
			product = productManager.getProductFromId(id);
		}
		catch(NumberFormatException e){
		}
		if(product.getState()==State.SOLD){
			return "payproduct";
		}
		else if(product.getState()==State.SENT){
			return "endsell";
		}
		else{
			return "";
		}
	}
	
	//"Select" a product to be acked
	//Return a redirection String
	public String actionSelectToBeAckedProduct(){
		try{
			int id = Integer.parseInt(getParam("product"));
			product = productManager.getProductFromId(id);
		}
		catch(NumberFormatException e){
			//TODO : return Faces Message
		}
		return "endsell";
	}
	
	//Pay a product
	//Return a redirection String
	public String actionPayProduct(){
		product.setState(State.PAID);
		productManager.modify(product);
		return "mybag";
	}
	
	//Close a sell
	//Return a redirection String
	public String actionCloseSell(){
		product.setState(State.RECEIVED);
		productManager.modify(product);
		return "mybag";
	}
	
	//Cf actionSelectSell
	//Return a different action depending of the user state (vendor/buyer)
	//Used in index.jsp...
	public String actionViewProduct(){
		product = new Product();
		try{
			int id = Integer.parseInt(getParam("product"));
			product = productManager.getProductFromId(id);
		}
		catch(NumberFormatException e){
		}
		if(product.getVendor().getLogin().equals(getUserController().getUser().getLogin())){
			return "editsell";
		}
		else{
			getBidController().setAmount(getBidController().actionGetBestBid(product).getAmount());
			return "newbid";
		}
	}

	//Edit a Sell
	//Return a redirection String
	public String actionEditSell(){
		productManager.modify(product);
		User user = getUserController().getUser();
		getUserController().setUser(userManager.getUser(user.getLogin(),user.getPassword()));
		return "mysell";
	}
	
	//Send a Product
	//Return a redirection String
	public String actionSendProduct(){
		product.setState(State.SENT);
		productManager.modify(product);
		return "mysell";
	}
	
	//Create a new Sell
	//Return a redirection String
	public String actionNewSell(){
		Date today = new Date();
		Date tommorow = new Date(today.getTime() + 86400000);
		setStartDate(today);
		setEndDate(tommorow);
		setQuantity(1);
		return "newsell";
	}
	
	//Get the list of the (my) sold products on which there are bids.
	public List<Product> getBiddedProducts(){
		productManager.updateProduct();
		List<Product> l = productManager.getSoldProductsInState(getUserController().getUser(), State.BID);
		return l;
	}
	
	//Get the list of the products on which the current user has a bid
	public List<Product> getBiddenProducts(){
		productManager.updateProduct();
		return productManager.getMyBiddenProductList(getUserController().getUser());
	}
	
	//Get the list of products the logged user have bought
	public List<Bid> getBoughtProducts(){
		 List<Bid> l = productManager.getMyBoughtProductList(getUserController().getUser());
		return l;
	}
	
	//Get the list of product the current user has to pay
	public List<Product> getToBePaidProductsBuyer(){
		List<Product> l = productManager.getBoughtProductsInState(getUserController().getUser(), State.SOLD);
		return l;
	}
	
	//Get the list of products the current user sold and that are not paid yet
	public List<Product> getToBePaidProductsVendor(){
		List<Product> l = productManager.getSoldProductsInState(getUserController().getUser(), State.SOLD);
		return l;
	}
	
	//Get the product paid by the logged User and not sent yet
	public List<Product> getToBeSentProductBuyer(){
		List<Product> l = productManager.getBoughtProductsInState(getUserController().getUser(), State.PAID);
		return l;
	}
	
	//The products the vendor has to send...
	public List<Product> getToBeSentProductVendor(){
		List<Product> l = productManager.getSoldProductsInState(getUserController().getUser(),State.PAID);
		return l;
	}
	
	//The products the buyer has to reveive...
	public List<Product> getToBeReceivedProductsBuyer(){
		List<Product> l = productManager.getBoughtProductsInState(getUserController().getUser(), State.SENT);
		return l;
	}
	
	//The products sent by the Vendor and not received yet by the buyer
	public List<Product> getToBeReceivedProductsVendor(){
		List<Product> l = productManager.getSoldProductsInState(getUserController().getUser(),State.SENT);
		return l;
	}
	
	//The list of product received by the user
	public List<Product> getReceived(){
		List<Product> l = productManager.getBoughtProductsInState(getUserController().getUser(), State.RECEIVED);
		return l;
	}
	
	//The list of sell over with or without buyer
	public List<Product> getEndedSells(){
		List<Product> l = productManager.getSoldProductsInState(getUserController().getUser(),State.RECEIVED);
		l.addAll(productManager.getSoldProductsInState(getUserController().getUser(),State.FINISHED));
		return l;
	}
	
	//Return an HTLM button, allowing the Vendor to send a product if, and only if, it has been paid
	public HtmlCommandButton getSendProductButton(){
		HtmlCommandButton b = new HtmlCommandButton();
		if(product.getState()!=State.PAID){
			b.setDisabled(true);
		}
		return b;
	}
	
	public void setSendProductButton(HtmlCommandButton b){
		
	}
	
	//Return the list of product the user is selling
	public List<Product> getMySells(){
		productManager.updateProduct();
		User user = getUserController().getUser();
		getUserController().setUser(userManager.getUser(user.getLogin(),user.getPassword()));
		return getUserController().getUser().getSells();
	}
	
	//Return the correct action name to be performed, given the product state and the current language
	public String getMessageFromState(Product p){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ResourceBundle bundle = ResourceBundle.getBundle("org.eBide.message.message", facesContext.getViewRoot().getLocale());
		if (p.getState()==State.BID){
			return bundle.getString("bid");
		}
		else if(p.getState()==State.SOLD){
			return bundle.getString("sold");
		}
		else if(p.getState()==State.PAID){
			return bundle.getString("paid");
		}
		else if(p.getState()==State.SENT){
			return bundle.getString("sent");
		}
		else if(p.getState()==State.RECEIVED){
			return bundle.getString("received");
		}
		else{
			return bundle.getString("finished");
		}
	}
	
	//getters and setters
	
	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getTitle() {
		return title;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getDescription() {
		return description;
	}

	public void setQuantity(int quantity) {
		this.quantity = quantity;
	}

	public int getQuantity() {
		return quantity;
	}

	public void setStartPrice(long startPrice) {
		this.startPrice = startPrice;
	}

	public long getStartPrice() {
		return startPrice;
	}

	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setCategoryId(int categoryId) {
		this.categoryId = categoryId;
	}

	public int getCategoryId() {
		return categoryId;
	}

}
