package org.eBide.controller;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;

import org.eBide.model.Product;
import org.openfaces.util.FacesUtil;

public class SearchController extends SuperController {
	private List<Product> products = new ArrayList<Product>();
	private List<Product> productList = new ArrayList<Product>();
	private Product selectedProduct;
	private Converter productConverter = new ProductConverter();
	
	//Load the list of required products
	//Return a redirection String
	public String search() {
		String title = "";
		if(selectedProduct != null && selectedProduct.getTitle() != null) {
			title = selectedProduct.getTitle();
		}
		products = productManager.lookForProduct(title.toLowerCase());
		return "searchresults";
	}

	//Load the list of products of the given category
	//Return a redirection String
	public String actionGetCategoriesProducts(){
		String catId = getParam("categoryID");
		int id = Integer.parseInt(catId);
		products = categoryManager.listProducts(id, false);
		return "searchresults";
	}
	
	// Used in the controller to create a dummy product
	private Product createProduct(String title) {
		Product p = new Product();
		p.setTitle(title);
		return p;
	}
	
	//Return the product whose title match the given String
	private Product getProductByTitle(String title){
		for(Product p : products) {
			if (p.getTitle().equals(title)) {
				return p;
			}
		}
		return null;
	}
	
	//getters and setters
	
	public void setProducts(List<Product> products) {
		this.products = products;
	}
	
	public List<Product> getProducts() {
		System.out.println(products);
		return products;
	}
	
	public void setProductList(List<Product> products) {
		this.productList = products;
	}

	public List<Product> getProductList() {
		String title = (String) FacesUtil.getRequestMapValue("searchString");
		if(title != null) {
			productList = productManager.lookForProduct(title.toLowerCase());
		}
		return productList;
	}
	
	public void setProductConverter(Converter productConverter) {
		this.productConverter = productConverter;
	}

	public Converter getProductConverter() {
		return productConverter;
	}

	public void setSelectedProduct(Product selectedProduct) {
		this.selectedProduct = selectedProduct;
	}

	public Product getSelectedProduct() {
		return selectedProduct;
	}

	// Used in the search Bar
	public class ProductConverter implements Converter, Serializable {

		private static final long serialVersionUID = -625243680892057730L;

		public Object getAsObject(FacesContext context, UIComponent component,
				String value) {
            if (value == null || value.equals(""))
                return null;
            Product product = getProductByTitle(value);
            if (product != null)
                return product;
            product = createProduct(value);
            return product;
		}

		public String getAsString(FacesContext context, UIComponent component,
				Object value) {
			if(value == null) {
				return "";
			}
			return ((Product)value).getTitle();
		}
		
	}

	
}
