package org.eBide.controller;


import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.servlet.http.HttpSession;

import org.eBide.model.Bid;
import org.eBide.model.Product;
import org.eBide.model.User;
import org.eBide.sessions.ManageUserRemote;

public class UserController extends SuperController{

	private User user;
	//Am i logged in?
	private boolean loggedIn;
	//Used to load a profile
	private String profile_login;
	//The current selected language
	private String lang;

	public UserController(){
		this.user = new User();
		this.loggedIn = false;
		// None-initialization of locale -> use of browser settings
	}

	//Change the current language to French
	public String actionSetLocalFR(){
		FacesContext context = FacesContext.getCurrentInstance();
		// Change locale of current page
		context.getViewRoot().setLocale(Locale.FRANCE);
		lang = "fr";
		return "";
	}
	
	//Change the current language to English
	public String actionSetLocalEN(){
		FacesContext context = FacesContext.getCurrentInstance();
		// Change locale of current page
		context.getViewRoot().setLocale(Locale.ENGLISH);
		lang = "en";
		return "";
	}

	//Get the 3 last sells of the user whose profile is requested
	public List<Product> getLastSellsProfile(){
		return productManager.getLastProductsSold(profile_login, 3);
	}
	
	//Get the 3 last bids of the user whose profile is requested
	public List<Bid> getLastBoughtProductsProfile(){
		return productManager.getMyBoughtProductList(profile_login, 3);
	}
	

	//Create a new User
	//Return a redirection String
	public String actionCreateUser(){
		user = userManager.addUser(user); 
		return actionLogin();
	}

	//Edit a profile
	//Return a redirection String
	public String actionEditProfile(){
		userManager.modify(user);
		return "editionOK";
	}

	//Log an user given his login/password
	//Return a redirection String
	public String actionLogin(){

		user = userManager.getUser(user.getLogin(), user.getPassword());

		if(user!=null){
			this.loggedIn = true;
			return "connectionOK";
		}
		else{
			return "connectionFailed";
		}
	}

	//Logout...
	//Return a redirectio String
	public String actionLogout(){

		FacesContext fc = FacesContext.getCurrentInstance();
		HttpSession session = (HttpSession) fc.getExternalContext().getSession(false);
		this.loggedIn = false;
		session.invalidate();
		return "loggedOut";
	}

	//Get a "M" or a "Mr" SelectItem, depending of the current language
	public SelectItem getMister(){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ResourceBundle bundle = ResourceBundle.getBundle("org.eBide.message.registration", facesContext.getViewRoot().getLocale());
		return new SelectItem(bundle.getString("m"));
	}

	//Cf get Mister...
	public SelectItem getMadam(){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ResourceBundle bundle = ResourceBundle.getBundle("org.eBide.message.registration", facesContext.getViewRoot().getLocale());
		return new SelectItem(bundle.getString("mrs"));
	}

	//Cf get Mister...
	public SelectItem getMiss(){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ResourceBundle bundle = ResourceBundle.getBundle("org.eBide.message.registration", facesContext.getViewRoot().getLocale());
		return new SelectItem(bundle.getString("miss"));
	}

	
	//Return a SelectItem table of country name, depending of the current language
	public SelectItem[] getCountryList(){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ResourceBundle bundle = ResourceBundle.getBundle("org.eBide.message.country", facesContext.getViewRoot().getLocale());
		return new SelectItem[] {
				new SelectItem(bundle.getString("andorra")),
				new SelectItem(bundle.getString("bahamas")),
				new SelectItem(bundle.getString("bahrain")),
				new SelectItem(bundle.getString("bermuda")),
				new SelectItem(bundle.getString("caiman")),
				new SelectItem(bundle.getString("islands")),
				new SelectItem(bundle.getString("liechtenstein")),
				new SelectItem(bundle.getString("luxembourg")),
				new SelectItem(bundle.getString("monaco")),
				new SelectItem(bundle.getString("sanmarino")),
				new SelectItem(bundle.getString("seychelles")),
				new SelectItem(bundle.getString("switzerland"))
		};
	}

	//Load my profile
	//Return a redirection String
	public String actionLoadProfile(){
		String login = getParam("user");

		profile_login = login;
		//		profile_photo = photo;
		return "userprofile";
	}

	//Return the pattern matching with the current language
	public String getPattern() {
		FacesContext context = FacesContext.getCurrentInstance();
		Locale locale = context.getViewRoot().getLocale();
		String pat = "MM/dd/yyyy";
		
		// If we have to deal with frenchies...
		if(locale.getLanguage().contains("fr")) {
			pat = "dd/MM/yyyy";
		}
		return pat;
	}
	
	
	//Getters and Setters
	
	public void setUser(User user) {
		this.user = user;
	}

	public User getUser() {
		return user;
	}

	public ManageUserRemote getUserManager() {
		return userManager;
	}

	public void setUserManager(ManageUserRemote userManager) {
		this.userManager = userManager;
	}

	public void setLoggedIn(boolean loggedIn) {
		this.loggedIn = loggedIn;
	}

	public boolean isLoggedIn() {
		return loggedIn;
	}

	public void setProfile_login(String profile_login) {
		this.profile_login = profile_login;
	}

	public String getProfile_login() {
		return profile_login;
	}

	public void setLang(String lang) {
		this.lang = lang;
	}

	public String getLang() {
		return lang;
	}

}
